use glob::glob;
use hotload_plugin::HotloadPlugin;
use std::env::current_exe;
use std::path::PathBuf;

pub struct PluginManager {
    pub plugins: Vec<*const HotloadPlugin>,
}

impl PluginManager {
    pub fn new() -> PluginManager {
        println!("Loading librairies...");
        let mut plugins = Vec::new();
        let current_path = current_exe().unwrap();
        let parent_path = current_path.parent().unwrap();
        let lib_folder = parent_path.join("*.so");
        for lib_file in glob(lib_folder.to_str().unwrap()).unwrap() {
            let plugin = Self::load_library(lib_file.unwrap());
            plugins.push(plugin);
        }

        PluginManager { plugins }
    }

    fn load_library(lib_file: PathBuf) -> *const HotloadPlugin {
        println!("Loading library {:?}", lib_file);
        unsafe {
            let lib = libloading::Library::new(lib_file).unwrap();
            let register_func: libloading::Symbol<extern "C" fn() -> *const HotloadPlugin> =
                lib.get(b"register").unwrap();
            register_func()
        }
    }
}
